/*
 * Copyright (C) 2013-2014 Canonical Ltd
 *
 * This file is part of Ubuntu Calendar App
 *
 * Ubuntu Calendar App is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 3 as
 * published by the Free Software Foundation.
 *
 * Ubuntu Calendar App is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

import QtQuick 2.4
import QtOrganizer 5.0
import Ubuntu.Components 1.3
//import Ubuntu.Components.ListItems 1.0 as ListItem
import "dateExt.js" as DateExt
import "./3rd-party/lunar.js" as Lunar

Page{
    id: root
    objectName: "AgendaView"

    property var currentDay: new Date()

    signal dateSelected(var date);

    Keys.forwardTo: [eventList]

    function goToBeginning() {
        eventList.positionViewAtBeginning();
    }

    function hasEnabledCalendars() {
        var enabled_calendars = eventListModel.getCollections().filter( function( item ) {
            return item.extendedMetaData( "collection-selected" );
        } );

        return !!enabled_calendars.length;
    }


    // Page Header
    header: PageHeader {
        title: i18n.tr("Agenda")
        leadingActionBar.actions: tabs.tabsAction
        trailingActionBar.actions: [
            commonHeaderActions.settingsAction,
            commonHeaderActions.newEventAction
        ]
        flickable: eventList
    }


    // ListModel to hold all events for upcoming 7days.
    EventListModel {
        id: eventListModel
        startPeriod: currentDay.midnight();
        endPeriod: currentDay.addDays(7).endOfDay()
        filter: eventModel.filter

        sortOrders: [
            SortOrder{
                blankPolicy: SortOrder.BlanksFirst
                detail: Detail.EventTime
                field: EventTime.FieldStartDateTime
                direction: Qt.AscendingOrder
            }
        ]
    }

    // spinner while agenda is loading.
    ActivityIndicator {
        visible: running
        running: eventListModel.isLoading
        anchors.centerIn: parent
        z:2
    }

    // Label to be showen when there is no upcoming events or if no calendar is selected.
    Label {
        id: noEventsOrCalendarsLabel
        text: !root.hasEnabledCalendars() ? i18n.tr("You have no calendars enabled") : i18n.tr( "No upcoming events" )
        visible: (eventListModel.itemCount === 0) && !eventListModel.isLoading
        anchors.centerIn: parent
    }

    // button to be showen when no calendar is selected (onClick will take user to list of all calendars)
    Button {
        text: i18n.tr( "Enable calendars" )
        visible: !root.hasEnabledCalendars()
        anchors.top: noEventsOrCalendarsLabel.bottom
        anchors.horizontalCenter: noEventsOrCalendarsLabel.horizontalCenter
        anchors.topMargin: units.gu( 1.5 )
        color: UbuntuColors.orange

        onClicked: {
            pageStack.push(Qt.resolvedUrl("CalendarChoicePopup.qml"),{"model":eventModel});
            pageStack.currentPage.collectionUpdated.connect(eventModel.delayedApplyFilter);
        }
    }

    // Main ListView with all upcoming events.
    ListView{
        id: eventList
        objectName: "eventList"
        model: eventListModel
        anchors.fill: parent
        visible: eventListModel.itemCount > 0

        delegate: listDelegate
    }

    // Scrollbar
    Scrollbar{
        flickableItem: eventList
        align: Qt.AlignTrailing
    }

    // ListView delegate
    Component{
        id: listDelegate

        // Main item to hold listitem delegate.
        Item {
            id: root
            property var event: eventListModel.items[index];

            width: parent.width
            height: container.height

            onEventChanged: {
                setDetails();
            }

            // function to set all detail in delegate (to be run on every eventListModel change.
            function setDetails() {
                if(event === null || event === undefined) {
                    return;
                }

                headerContainer.visible = false; // hide headerContainer
                if( index == 0 ) {
                    headerContainer.visible = true; // if this is first event in that date then show headerContainer
                } else {
                    var prevEvent = eventListModel.items[index-1];
                    if( prevEvent.startDateTime.midnight() < event.startDateTime.midnight()) {
                        headerContainer.visible = true;
                    }
                }

                var date = event.startDateTime.toLocaleDateString()
                var startTime = event.startDateTime.toLocaleTimeString(Qt.locale(), Locale.ShortFormat)
                var endTime = event.endDateTime.toLocaleTimeString(Qt.locale(), Locale.ShortFormat)

                // TRANSLATORS: the first argument (%1) refers to a start time for an event,
                // while the second one (%2) refers to the end time
                var timeStringStart = i18n.tr("%1").arg(startTime) + " -"
                var timeStringEnd = i18n.tr("%1").arg(endTime)

                if (mainView.displayLunarCalendar) {
                    var lunarDate = Lunar.calendar.solar2lunar(event.startDateTime.getFullYear(),
                                                               event.startDateTime.getMonth() + 1,
                                                               event.startDateTime.getDate())
                    header.text = i18n.tr("%1 %2 %3 %4 %5").arg(lunarDate.gzYear).arg(lunarDate .IMonthCn).arg(lunarDate.IDayCn)
                                                            .arg(lunarDate.gzDay).arg(lunarDate.isTerm ? lunarDate.Term : "")
                } else {
                    header.text = date.charAt(0).toUpperCase() + date.slice(1);
                }

                timeLabelStart.text = timeStringStart
                timeLabelEnd.text = timeStringEnd
                header.color = event.startDateTime.toLocaleDateString() === new Date().toLocaleDateString() ? UbuntuColors.orange : UbuntuColors.darkGrey
                calendarIndicator.backgroundColor = eventListModel.collection(event.collectionId).color

                if(event.displayLabel) {
                    eventLabel.text = i18n.tr("%1").arg(event.displayLabel);
                }
                if(event.location) {
                    locationLabel.text = i18n.tr("%1").arg(event.location);
                    locationLabel.color = Theme.palette.selected.backgroundText
                    locationLabel.font.italic = false
                } else {
                    locationLabel.text = i18n.tr("no location")
                    locationLabel.color = "#b3b3b3"
                    locationLabel.font.italic = true
                }
            }



            // main Column to hold   header (date) and details (time, Description and location)
            Column {
                id: container
                objectName: "eventContainer" + index

                width: parent.width
                anchors.top: parent.top

                // Header ListItem eg. ( Friday, October 29th 2015 )
                ListItem {
                    id: headerContainer
                    height: visible ? header.height + units.gu(3) : 0
                    width: parent.width
                    // ListItem background (changing color when pressed)
                    Rectangle {
                        anchors.fill: parent
                        color: headerMouseArea.pressed ? "#ededed" : "#f7f7f7"
                    }
                    // Label to hold string with date eg. ( Friday, October 29th 2015 )
                    Label{
                        id: header
                        fontSize: "medium"
                        width: parent.width
                        elide: Text.ElideRight
                        anchors {
                            left: parent.left
                            leftMargin: units.gu(2)
                            verticalCenter: parent.verticalCenter
                        }
                    }
                    // MouseArea, onClicked new page with dayle view will open.
                    MouseArea {
                        id: headerMouseArea
                        anchors.fill: parent
                        onClicked: {
                            Haptics.play()
                            dateSelected(event.startDateTime);
                        }
                    }

                }

                // Main ListItem to hold Details about event eg. ( 19:30 -   Beer with the team )
                //                                               ( 20:00     Hicter             )
                ListItem {
                    id: detailsListItem
                    width: parent.width
                    height: detailsColumn.height + units.gu(4)

                    // ListItem background (changing color when pressed)
                    Rectangle {
                        anchors.fill: parent
                        color: detailsMouseArea.pressed ? "#f7f7f7" : "white"
                    }

                    // Little icon in left top corner of every event to indicate color of the calendar.
                    UbuntuShape {
                        id: calendarIndicator
                        anchors {
                            left: parent.left;
                            leftMargin: units.gu(1)
                            top: parent.top;
                            topMargin: units.gu(1)
                        }
                        width: units.gu(1)
                        height: width
                        aspect: UbuntuShape.DropShadow
                    }

                    // main Column to hold Grid with times, Description and location.
                    Column{
                        id: detailsColumn
                        anchors {
                            left: parent.left
                            right: parent.right
                            verticalCenter: parent.verticalCenter
                        }

                        // Mian Grid
                        Grid {
                            columns: 2
                            columnSpacing: units.gu(1.5)
                            anchors {
                                left: parent.left
                                leftMargin: units.gu(3)
                                right: parent.right
                            }

                            // Event time start
                            Label{
                                id: timeLabelStart
                                objectName: "timeLabelStart" + index
                                color: UbuntuColors.coolGrey
                                fontSize: "small"
                            }
                            // Event descriptions
                            Label{
                                id: eventLabel
                                objectName: "eventLabel" + index
                                width: parent.width - timeLabelStart.width - parent.columnSpacing
                                elide: Text.ElideRight
                                color: UbuntuColors.coolGrey
                                font.bold: true
                                fontSize: "small"
                            }

                            // Event time End
                            Label{
                                id: timeLabelEnd
                                objectName: "timeLabelEnd" + index
                                color: UbuntuColors.coolGrey
                                fontSize: "small"
                            }

                            // Event Location
                            Label{
                                id: locationLabel
                                objectName: "locationLabel" + index
                                width: parent.width - timeLabelStart.width - parent.columnSpacing
                                elide: Text.ElideRight
                                color: UbuntuColors.coolGrey
                                fontSize: "small"
                            }
                        }


                    }

                    // MouseArea, onClicked open new edit event page.
                    MouseArea {
                        id: detailsMouseArea
                        anchors.fill: parent
                        onClicked: {
                            Haptics.play()
                            pageStack.push(Qt.resolvedUrl("EventDetails.qml"), {"event":event,"model":eventListModel});
                        }
                    }
                }

            }
        }
    }
}
